import traceback, sys, os, pyRXPU
_logf = None
_bad = 0
_total = 0
def _dot(c,write=sys.stdout.write):
	global _total, _bad
	write(c)
	if c=='E': _bad = _bad + 1
	if _total%40 == 39: write('\n')
	_total = _total + 1

greeks = {
	'Aacute': b'\xc3\x81',
	'aacute': b'\xc3\xa1',
	'Acirc': b'\xc3\x82',
	}
def ugeCB(name):
	#plogn('ugeCB called with name=%r' % name)
	try:
		return greeks[name]
	except:
		return '&#38;'+name+';'

def plog(x):
	_logf.write(x+'\n')
	_logf.flush()

def plogn(x):
	plog(x +('' if x.endswith('\n') else '\n'))

def goodTest(x,t,tb=0,inOnly=0,**kw):
	try:
		P=pyRXPU.Parser(**kw)
		r = P(x)
		rb = 0
	except:
		et, ev, _unused = sys.exc_info()
		sev = ascii(ev)
		sev = sev[6:-3 if sev.endswith(",)") else -2]
		if sev.startswith('u'): sev = sev[1:]
		r = '%s %s' % (et.__name__, sev[1:])
		rb = 1

	s = ''
	for k,v in kw.items():
		s = s+', %s=%s' % (k,str(v))
	if type(t) is type(''):
		t = t.replace('\r','\\r')
		t = t.replace('\n','\\n')
	if type(r) is type(''):
		r = r.replace('\r','\\r')
		r = r.replace('\n','\\n')
	plog('%s.Parser(%s)(%s)'%(pyRXPU.__name__,s[2:],repr(x)))
	if (inOnly and t in r) or (r==t) and rb==tb:
		plogn('OK')
		_dot('.')
	else:
		_dot('E')
		plogn('\nBAD got %s' % repr(r))
		plogn('Expected %s' % repr(t))

def failTest(x,t,tb=1,inOnly=0,**kw):
	goodTest(x,t,tb,inOnly=inOnly,**kw)

def bigDepth(n):
	return n and '<tag%d>%s</tag%d>' % (n,bigDepth(n-1),n) or 'middle'

def getHere():
	if __name__=='__main__':
		here = sys.argv[0]
	else:
		here = __file__
	here = os.path.dirname(here)
	cwd = os.getcwd()
	if not here:
		here = cwd
	else:
		try:
			os.chdir(here)
			here = os.getcwd()
		finally:
			os.chdir(cwd)
	return here

with open(os.path.join(getHere(),'is-there.dtd'),'rb') as f:
	have_utf8_content_dtd = f.read()
have_unicode_content_dtd = have_utf8_content_dtd.decode('utf8')

DTDs = 	{
		'is-there.dtd': 'is-there.dtd',
		'not-there.dtd': 'really-not-there.dtd',
		'have-utf8-content.dtd': ('have-utf8-content.dtd-uri',have_utf8_content_dtd),
		'have-unicode-content.dtd': ('have-unicode-content.dtd-uri',have_unicode_content_dtd),
		'badt-have-utf8-content.dtd': (1,have_utf8_content_dtd),
		'badt-have-unicode-content.dtd': (1,have_unicode_content_dtd),
		}

def eoDTD(s):
	return DTDs.get(os.path.basename(s),None)

def _runTests():
	plogn('############# Testing %s=%8.8X'%(pyRXPU.__name__,id(pyRXPU)))
	try:
		for k,v in pyRXPU.parser_flags.items(): eval('pyRXPU.Parser(%s=%d)' % (k,v))
		plogn('Parser keywords OK')
		_dot('.')
	except:
		traceback.print_exc()
		plogn('Parser keywords BAD')
		_dot('E')
	try:
		for k,v in pyRXPU.parser_flags.items(): eval('pyRXPU.Parser()("<a/>",%s=%d)' % (k,v))
		plogn('Parser().parse keywords OK')
		_dot('.')
	except:
		traceback.print_exc()
		plogn('Parser().parse keywords BAD')
		_dot('E')

	try:
		P=pyRXPU.Parser()
		plog('Parser()=%r' % P)
		plog('Parser().__class__=%r' % P.__class__)
		plog('type(Parser())=%r\n\n' % type(P))
		del P
		plogn('Parser().__class__ etc OK')
		_dot('.')
	except:
		traceback.print_exc()
		plogn('Parser().__class__ etc BAD')
		_dot('E')

	goodTest('<a></a>',('a', None, [], None))
	goodTest('<a></a>',('a', {}, [], None),ExpandEmpty=1)
	goodTest('<a></a>',['a', None, [], None],MakeMutableTree=1)
	goodTest('<a/>',('a', None, None, None))
	goodTest('<a/>',('a', {}, [], None),ExpandEmpty=1)
	goodTest('<a/>',['a', None, None, None],MakeMutableTree=1)
	goodTest('<a/>',['a', {}, [], None],ExpandEmpty=1,MakeMutableTree=1)
	failTest('</a>',"error Error: End tag </a> outside of any element\n in unnamed entity at line 1 char 4 of [unknown]\nEnd tag </a> outside of any element\nParse Failed!\n")
	goodTest('<a>A<!--comment--></a>',('a', None, ['A'], None))
	goodTest('<a>A<!--comment--></a>',('a', {}, ['A'], None),ExpandEmpty=1)
	goodTest('<a>A<!--comment--></a>', ('a', None, ['A', ('<!--', None, ['comment'], None)], None), ReturnComments=1)
	goodTest('<a>A&lt;&amp;&gt;</a>',('a', None, ['A<&>'], None))
	goodTest('<a>A&lt;&amp;&gt;</a>',('a', None, ['A', '<', '&', '>'], None), MergePCData=0)
	goodTest('<!--comment--><a/>',('a', None, None, None),ReturnComments=1)
	goodTest('<!--comment--><a/>',[('<!--',None,['comment'],None),('a', None, None, None)],ReturnComments=1,ReturnList=1)
	goodTest('<!--comment--><a/>',('a', None, None, None),ReturnComments=1)
	failTest('<?xml version="1.0" encoding="LATIN-1"?></a>',"error Unknown declared encoding LATIN-1\nInternal error, ParserPush failed!\n")
	goodTest('<?work version="1.0" encoding="utf-8"?><a/>',[('<?',{'name':'work'}, ['version="1.0" encoding="utf-8"'],None), ('a', None, None, None)],IgnorePlacementErrors=1,ReturnList=1,ReturnProcessingInstructions=1,ReturnComments=1)
	goodTest('<a>\nHello\n<b>cruel\n</b>\nWorld\n</a>',('a', None, ['\nHello\n', ('b', None, ['cruel\n'], (('aaa', 2, 3), ('aaa', 3, 4))), '\nWorld\n'], (('aaa', 0, 3), ('aaa', 5, 4))),fourth=pyRXPU.recordLocation,srcName='aaa')
	goodTest('<a aname="ANAME" aother="AOTHER">\nHello\n<b bname="BNAME" bother="BOTHER">cruel\n</b>\nWorld\n</a>',('a', {"aname": "ANAME", "aother": "AOTHER"}, ['\nHello\n', ('b', {"bname": "BNAME", "bother": "BOTHER"}, ['cruel\n'], (('aaa', 2, 33), ('aaa', 3, 4))), '\nWorld\n'], (('aaa', 0, 33), ('aaa', 5, 4))),fourth=pyRXPU.recordLocation,srcName='aaa')
	goodTest('<a><![CDATA[<a>]]></a>',('a', None, ['<a>'], None))
	goodTest('<a><![CDATA[<a>]]></a>',('a', None, [('<![CDATA[', None, ['<a>'], None)], None),ReturnCDATASectionsAsTuples=1)
	goodTest('''<foo:A xmlns:foo="http://www.foo.org/"><foo:B><foo:C xmlns:foo="http://www.bar.org/"><foo:D>abcd</foo:D></foo:C></foo:B><foo:B/><A>bare A<C>bare C</C><B>bare B</B></A><A xmlns="http://default.reportlab.com/" xmlns:bongo="http://bongo.reportlab.com/">default ns A<bongo:A>bongo A</bongo:A><B>default NS B</B></A></foo:A>''',('{http://www.foo.org/}A', {'xmlns:foo': 'http://www.foo.org/'}, [('{http://www.foo.org/}B', None, [('{http://www.bar.org/}C', {'xmlns:foo': 'http://www.bar.org/'}, [('{http://www.bar.org/}D', None, ['abcd'], None)], None)], None), ('{http://www.foo.org/}B', None, None, None), ('A', None, ['bare A', ('C', None, ['bare C'], None), ('B', None, ['bare B'], None)], None), ('{http://default.reportlab.com/}A', {'xmlns': 'http://default.reportlab.com/', 'xmlns:bongo': 'http://bongo.reportlab.com/'}, ['default ns A', ('{http://bongo.reportlab.com/}A', None, ['bongo A'], None), ('{http://default.reportlab.com/}B', None, ['default NS B'], None)], None)], None),XMLNamespaces=1,ReturnNamespaceAttributes=1)
	failTest(bigDepth(257),"""error Internal error, stack limit reached!\n""", inOnly=1)
	failTest('<a>&Aacute;&aacute;</a>','error Error: Undefined entity Aacute\n in unnamed entity at line 1 char 12 of [unknown]\nUndefined entity Aacute\nParse Failed!\n')
	goodTest('<a>&Aacute;</a>',('a', None, ['\xc1'], None), ugeCB=ugeCB)
	filename = os.path.join(os.getcwd(),'not-there.dtd').replace(os.sep,'/')
	if filename.startswith('/'): filename = filename[1:]
	failTest('<!DOCTYPE foo SYSTEM "not-there.dtd"><foo>foo<a>aaa</a>fum</foo>',"error Error: Couldn't open dtd entity file:///%(filename)s\\n in unnamed entity at line 1 char 38 of [unknown]"%vars(),NoNoDTDWarning=0,inOnly=1)
	failTest('<!DOCTYPE foo SYSTEM "is-there.dtd"><foo><a>aaa</a></foo>','error Error: Content model for foo does not allow it to end here\\n in unnamed entity at line 1 char 57 of [unknown]',inOnly=1,NoNoDTDWarning=0)
	goodTest('<!DOCTYPE foo SYSTEM "is-there.dtd"><foo><a>aaa</a><b>bbbb</b></foo>',('foo', None, [('a', None, ['aaa'], None), ('b', None, ['bbbb'], None)], None),NoNoDTDWarning=0)
	failTest('<!DOCTYPE foo SYSTEM "is-there.dtd"><foo><a>aaa</a></foo>','error Error: Content model for foo does not allow it to end here',inOnly=1,NoNoDTDWarning=0,eoCB=eoDTD)
	goodTest('<!DOCTYPE foo SYSTEM "is-there.dtd"><foo><a>aaa</a><b>bbbb</b></foo>',('foo', None, [('a', None, ['aaa'], None), ('b', None, ['bbbb'], None)], None),NoNoDTDWarning=0,eoCB=eoDTD)
	goodTest('<!DOCTYPE foo SYSTEM "have-utf8-content.dtd"><foo><a>aaa</a><b>bbbb</b></foo>',('foo', None, [('a', None, ['aaa'], None), ('b', None, ['bbbb'], None)], None),NoNoDTDWarning=0,eoCB=eoDTD)
	goodTest('<!DOCTYPE foo SYSTEM "have-unicode-content.dtd"><foo><a>aaa</a><b>bbbb</b></foo>',('foo', None, [('a', None, ['aaa'], None), ('b', None, ['bbbb'], None)], None),NoNoDTDWarning=0,eoCB=eoDTD)
	filename = os.path.join(os.getcwd(),'really-not-there.dtd').replace(os.sep,'/')
	if filename.startswith('/'): filename = filename[1:]
	failTest('<!DOCTYPE foo SYSTEM "not-there.dtd"><foo>foo<a>aaa</a>fum</foo>',"error Error: Couldn't open dtd entity file:///%(filename)s\\n in unnamed entity at line 1 char 38 of [unknown]\\n"%vars(),NoNoDTDWarning=0,eoCB=eoDTD,inOnly=1)
	failTest('<!DOCTYPE foo SYSTEM "badt-have-utf8-content.dtd"><foo><a>aaa</a><b>bbbb</b></foo>',"error Error: Couldn't open dtd entity badt-have-utf8-content.dtd\\n in unnamed entity at line 1 char 51 of [unknown]",inOnly=1,NoNoDTDWarning=0,eoCB=eoDTD)
	failTest('<!DOCTYPE foo SYSTEM "badt-have-unicode-content.dtd"><foo><a>aaa</a><b>bbbb</b></foo>',"error Error: Couldn't open dtd entity badt-have-unicode-content.dtd\\n in unnamed entity at line 1 char 54 of [unknown]",inOnly=1,NoNoDTDWarning=0,eoCB=eoDTD)

def main():
	if '__doc__' in sys.argv:
		print(pyRXPU.__doc__)
	else:
		global _logf
		with open('pyRXP_test.log','w') as _logf:
			_runTests()
			msg = ("\n%d tests" % _total) + (', %d FAILED!' % _bad if _bad else '\n\nOK')
			print(msg)
			plogn(msg)
			if _bad:
				sys.exit('failure in testRXPBasic')

if __name__=='__main__': #noruntests
	main()



def test1(n):
	print('starting test1')
	import pyRXPU, sys
	xml='<start><tag1>text here</tag1><tag1>more text</tag1></start>'
	P = i = tup = None
	for i in range(n):
		P=pyRXPU.Parser()
		tup=P(xml)
		sys.stdout.write('.')
	del n, pyRXPU, sys, P, tup, xml
	print('\ntest1 done')

def test2(n):
	import os
	src = """<!DOCTYPE p SYSTEM "xhtml1-strict.dtd">
	<p>The York Early Music Christmas Festival has been going
strong since 1997 and in 2005 sees guest artists such as the Orlando
Consort, the Bach Players, Joglaresa, the Gonzaga Band and the
Huddersfield University Early Music Ensemble. Venues include the
Chapter House of York Minster and St Olave's Church, Marygate as
well as the National Centre for Early Music itself.</p>"""

	def find_dtd():
		try:
			import rlextra
		except ImportError:
			return os.path.join(os.path.dirname(__file__),'xhtml1')
		else:
			return os.path.join(rlextra.__path__[0],'dtd','xhtml1')

	def ident(x,dtdDir=find_dtd(),path_isfile=os.path.isfile,path_join=os.path.join):
		bn = x.split('/')[-1]
		dtd = path_join(dtdDir,bn)
		if path_isfile(dtd):
			return (x, open(dtd).read())
		return x

	def run2():
		import pyRXPU, sys
		p = pyRXPU.Parser(eoCB = ident)
		for i in range(n):
			tree = p.parse(src)
			sys.stdout.write('.')

	run2()
	print('\ntest2 done')

def main(N=None):
	import sys
	n = [x for x in sys.argv if x.startswith('--test=')]
	tests = n and [int(x.lstrip('--test=')) for x in n] or [1,2]
	map(sys.argv.remove,n)
	if N is None:
		if len(sys.argv)>1: N = int(sys.argv[1])
		else: N = 10000
	if 1 in tests:
		test1(N)
	if 2 in tests:
		test2(N)

if __name__=='__main__': #noruntests
	main()



import sys, os, pprint, time
import pyRXPU as pyRXP

def test(fn):
	cwd = os.getcwd()
	try:
		print('BEGIN')
		t0 = time.time()
		xml = open(fn,'r').read()
		os.chdir(os.path.dirname(fn))
		t1 = time.time()
		D = pyRXP.Parser().parse(xml)
		t2 = time.time()
		print('END read took %7.2f" parse took %7.2f"' %(t1-t0,t2-t1))
		return D
	finally:
		os.chdir(cwd)
	
def main():
	def Usage():
		print('Usage %s [-n] filename' % sys.argv[0], file=sys.stderr)
		sys.exit()
	if len(sys.argv)<2: Usage()
	nopp = sys.argv[1]=='-n'
	if nopp: del sys.argv[1]
	if len(sys.argv)<2: Usage()
	fn = sys.argv[1]
	D = test(fn)
	if not nopp: pprint.pprint(D)

if __name__=='__main__':
	main()



#!/usr/bin/env python
from __future__ import print_function
'''
$Id: test_xmltestsuite.py,v 1.2 2003/04/13 16:04:04 rgbecker Exp $
Test parsing and validation against James Clark's test cases,
as downloaded from http://www.jclark.com/xml/
The .zip file should be in the same directory as this script.
Note that the .zip file can be freely distributed in unmodified form
so it could be added to the pyRXP distribution.
'''
__rcs_id__	= '$Id: test_xmltestsuite.py,v 1.2 2003/04/13 16:04:04 rgbecker Exp $'
__version__ = '$Revision: 1.2 $'[11:-2]
__author__ = 'Stuart Bishop <stuart@stuartbishop.net>'

import unittest, zipfile, sys, os, os.path, codecs
debug = int(os.environ.get('DEBUG_PYRXP','0'))
import pyRXPU

# Debug is to help me trace down memory bugs
if debug: import time

class test_pyRXPU(unittest.TestCase):

	def parse(self,filename,**kw):
		if debug&2: print('##### About to parse %s' % filename, file=sys.stderr)
		kw = kw.copy()
		kw['ReturnComments'] = 0
		kw['ExpandEmpty'] = 1
		kw['XMLLessThan'] = 1
		kw['ReturnProcessingInstructions'] = 1
		kw['ReturnList'] = 1
		if debug&4:
			def eocb(s):
				print(f'+++++ eocb({s!r})',file=sys.stderr,flush=True)
				return s
			kw['eoCB'] = eocb
		parser = pyRXPU.Parser(**kw)
		# Change directory in case we are loading entities from cwd
		retdir = os.getcwd()
		d,n = os.path.split(filename)
		os.chdir(d)
		try:
			with open(n,'rb') as f:
				xml = f.read()
			r = parser.parse(xml)
			if debug:
				print(f'##### r={ascii(r)}',file=sys.stderr,flush=True)
			return r
		finally:
			os.chdir(retdir)
			if debug&2: print('Done parsing   %s' % filename, file=sys.stderr,flush=True)
			if debug&2: print('='*60, file=sys.stderr,flush=True)
			if debug&16: time.sleep(1)

	def getcanonical(self,filename):
		''' Parse in the named file, and return it as canonical XML '''
		return self._getcan(self.parse(filename))

	def _getcan(self,node):
		if type(node) is type([]): return ''.join(map(self._getcan, node))
		if type(node) in (type(''),type(u'')):
			return self._quote(node)

		tag,attrs,kids,junk = node

		if tag == pyRXPU.commentTagName:
			return u'<!--%s-->' % (kids[0])
		elif tag == pyRXPU.piTagName:
			return u'<?%s %s?>' % (attrs['name'],kids[0])

		if attrs is None:
			attrs = ''
		else:
			keys = list(attrs.keys())
			keys.sort() # Attributes in lexical order
			attrs = ' '.join(
				['%s="%s"' % (k,self._quote(attrs[k])) for k in keys]
				)
			if attrs:
				attrs = ' ' + attrs

		text = ''.join([self._getcan(kid) for kid in kids])

		return '<%s%s>%s</%s>' % (tag,attrs,text,tag)

	def _quote(self,txt):
		txt = txt.replace('&','&amp;')
		txt = txt.replace('<','&lt;')
		txt = txt.replace('>','&gt;')
		txt = txt.replace('"','&quot;')
		txt = txt.replace('\x09','&#9;')
		txt = txt.replace('\x0a','&#10;')
		txt = txt.replace('\x0d','&#13;')
		return txt

	def _test_valid(self,inname,outname):
		inxml = self.getcanonical(inname)
		with codecs.open(outname,mode='rb',encoding='utf8') as f:
			outxml = f.read()
		self.assertEqual(inxml,outxml,'%s != %s' % (inname,outname))
		if debug&8: print(f'##### _test_valid({inname!r}) OK')  

	def _test_invalid_parse(self,inname):
		try:
			self.parse(inname,Validate=0)
		except pyRXPU.error:
			if debug&8: print(f'##### _test_invalid_parse({inname!r}) OK')  

	def _test_invalid_validate(self,inname):
		try:
			self.parse(inname,Validate=1)
			self.fail('!!!!! Failed to detect validity error in %r' % inname)
		except pyRXPU.error:
			if debug&8: print(f'##### _test_invalid_validate({inname!r}) OK')  

	def _test_notwf(self,inname):
		try:
			self.parse(inname,Validate=0)
			self.fail(
				'!!!!! Failed to detect that %r was not well formed' % inname
				)
		except pyRXPU.error:
			if debug&8: print(f'##### _test_notwf({inname!r}) OK')  

def buildup_test(cls=test_pyRXPU,I=[]):
	''' Add test methods to the TestCase '''
	cls.valid = []
	cls.invalid = []
	cls.notwf = []
	testdir = os.path.dirname(__file__)
	try:
		zipfName = os.path.join(testdir,'xmltest.zip')
		zipf = zipfile.ZipFile(zipfName)
	except:
		print("!!!!! Can't locate file xmltest.zip\nPerhaps it should be downloaded from\nhttp://www.reportlab.com/ftp/xmltest.zip\nor\nftp://ftp.jclark.com/pub/xml/xmltest.zip\n", file=sys.stderr)
		raise
	else:
		if debug&8:
			print(f'##### obtained {zipf!r} from {zipfName!r}') 

	for zipname in sorted(zipf.namelist()):
		#skip broken tests
		#if sys.platform=='win32':
		#	if zipname in ('xmltest/valid/ext-sa/014.xml',
		#			'xmltest/valid/ext-sa/014.ent',
		#			'xmltest/valid/ext-sa/out/014.xml'):
		#		continue
		# Extract the files if they don't alrady exist
		osname = os.path.join(*zipname.split('/')) # For non-unixes
		osname = os.path.join(testdir,osname)
		dir = os.path.dirname(osname)
		if not os.path.isdir(dir):
			os.makedirs(dir)
		if not os.path.isfile(osname):
			if debug&8: print(f'##### creating {osname!r}...',end='...')
			with open(osname,'wb') as f:
				f.write(zipf.read(zipname))
			if debug&8: print('written',flush=True)
		if I and zipname not in I: continue

		# Add input files to our lists
		if os.path.splitext(osname)[1] == '.xml' and zipname.find('out') == -1:
			if zipname.find('invalid') != -1:
				cls.invalid.append(osname)
			elif zipname.find('not-wf') != -1:
				cls.notwf.append(osname)
			elif zipname.find('valid') != -1:
				outname = os.path.join(dir,'out',os.path.basename(osname))
				cls.valid.append( (osname,outname) )

	if debug&8:
		print(f'##### |{cls!r}.valid| = {len(cls.valid)}',flush=True)
		print(f'##### |{cls!r}.invalid| = {len(cls.invalid)}',flush=True)
		print(f'##### |{cls!r}.notwf| = {len(cls.notwf)}',flush=True)

	# Add 'valid' tests
	for inname,outname in cls.valid:
		num = int(os.path.splitext(os.path.basename(inname))[0])
		dir = os.path.split(os.path.split(inname)[0])[1]
		mname = ('test_Valid_%s_%03d' % (dir,num)).replace('-','_')
		def doTest(self,inname=inname,outname=outname):
			self._test_valid(inname,outname)
		setattr(cls,mname,doTest)
	if debug&8: print('##### valid tests created')

	# Add 'invalid' tests
	for inname in cls.invalid:
		num = int(os.path.splitext(os.path.basename(inname))[0])
		mname = 'test_InvalidParse_%03d' % (num)
		def doTest(self,inname=inname):
			self._test_invalid_parse(inname)
		setattr(cls,mname,doTest)
		mname = 'test_InvalidValidate_%03d' % (num)
		def doTest(self,inname=inname):
			self._test_invalid_validate(inname)
		setattr(cls,mname,doTest)
	if debug&8: print('##### invalid tests created')

	# Add 'not wellformed' tests
	for inname in cls.notwf:
		num = int(os.path.splitext(os.path.basename(inname))[0])
		dir = os.path.split(os.path.split(inname)[0])[1]
		mname = ('test_NotWellFormed_%s_%03d' % (dir,num)).replace('-','_')
		def doTest(self,inname=inname):
			self._test_notwf(inname)
		setattr(cls,mname,doTest)
	if debug&8: print('##### notwf tests created')

def makeTests(debug=0):
	globals()['debug'] = debug
	I = filter(lambda a: a[:2]=='-I',sys.argv)
	for i in I: sys.argv.remove(i)
	I = list(map(lambda x: x[2:],I))
	buildup_test(I=I)

def main(verbose=0, singles=0):
	makeTests(debug=debug)
	if singles:
		T = sorted([t for t in test_pyRXPU.__dict__ if t.startswith("test_")])
		for t in T:
			print(f'+++++ test_xmltestsuite.test_pyRXPU.{t}',end='.....',flush=True)
			r = os.system(' '.join((sys.executable, 'test_xmltestsuite.py', f'test_pyRXPU.{t}')))
			if r:
				print(f'FAIL !!!!! {t} exited with error {r} !!!!!')
			else:
				print(' OK')
	else:
		unittest.main(module=__name__,exit=False)

if __name__ == '__main__':
	main()



class Unbuffered(object):
    def __init__(self, stream):
        self.stream = stream
    def write(self, data):
        self.stream.write(data)
        self.stream.flush()
    def writelines(self, datas):
        self.stream.writelines(datas)
        self.stream.flush()
    def __getattr__(self, attr):
        return getattr(self.stream, attr)

def main():
    import sys, os
    dbg = int(os.environ.get('PYRXPDEBUG','0'))
    sys.__old_stderr__ = sys.stderr
    sys.__stderr__ = sys.stderr = sys.stdout = Unbuffered(sys.stdout)
    wd = os.path.dirname(os.path.abspath(sys.argv[0]))
    sys.path.insert(0,wd)
    os.chdir(wd)
    if dbg:
        import psutil, platform
        platform_attrs = [
            "architecture",
            "java_ver",
            "libc_ver",
            "machine",
            "mac_ver",
            "node",
            "platform",
            "processor",
            "python_branch",
            "python_build",
            "python_compiler",
            "python_implementation",
            "python_revision",
            "python_version",
            "python_version_tuple",
            "release",
            "system",
            "system_alias",
            "uname",
            "version",
            "win32_edition",
            "win32_is_iot",
            "win32_ver",
            ]
        print(f'+++++ {sys.platform}')
        print(f'+++++ {sys.byteorder}')
        for a in platform_attrs:
            if hasattr(platform,a):
                try:
                    print(f'+++++ platform.{a}()={getattr(platform,a)()}')
                except:
                    print(f'!!!!! platform.{a}()=cannot be determined')

    #special case import to allow reportlab to modify the environment in development
    try:
      import reportlab
    except ImportError:
      pass

    import leaktest, testRXPbasic, test_xmltestsuite

    leaktest.main(100)
    testRXPbasic.main()
    if 1 or platform.system()!='Darwin':
        verbose=int(os.environ.get('VERBOSE','0'))
        singles=int(os.environ.get('SINGLES','0'))
        test_xmltestsuite.main(verbose, singles)
    else:
        test_xmltestsuite.main(3,1)
    if dbg:
        print(f'+++++ open Files={psutil.Process().open_files()!r}')

if __name__=='__main__':
    main()
